// http://cs.brown.edu/people/pfelzens/papers/dt-final.pdf
/**
 * Takes `ImageData` and returns a `new ImageData()` with the SDF applied.
 */
export function toSDF(imageData, width, height, radius) {
    const gridOuter = new Float64Array(width * height);
    const gridInner = new Float64Array(width * height);
    const INF = 1e20;
    for (let i = 0; i < width * height; i++) {
        const a = imageData.data[i * 4 + 3] / 255; // Alpha value.
        if (a === 1) {
            gridOuter[i] = 0;
            gridInner[i] = INF;
        }
        else if (a === 0) {
            gridOuter[i] = INF;
            gridInner[i] = 0;
        }
        else {
            const d = 0.5 - a;
            gridOuter[i] = d > 0 ? d * d : 0;
            gridInner[i] = d < 0 ? d * d : 0;
        }
    }
    const size = Math.max(width, height);
    const f = new Float64Array(size);
    const z = new Float64Array(size + 1);
    const v = new Uint16Array(size * 2);
    edt(gridOuter, width, height, f, v, z);
    edt(gridInner, width, height, f, v, z);
    const alphaChannel = new Uint8ClampedArray(width * height);
    for (let i = 0; i < width * height; i++) {
        const d = Math.sqrt(gridOuter[i]) - Math.sqrt(gridInner[i]);
        alphaChannel[i] = Math.round(255 - 255 * (d / radius + 0.25));
    }
    const data = new Uint8ClampedArray(width * height * 4);
    for (let i = 0; i < width * height; i++) {
        data[4 * i + 0] = alphaChannel[i];
        data[4 * i + 1] = alphaChannel[i];
        data[4 * i + 2] = alphaChannel[i];
        data[4 * i + 3] = alphaChannel[i];
    }
    return new ImageData(data, width, height);
}
const INF = 1e20;
// 1D squared distance transform.
function edt1d(grid, offset, stride, length, f, v, z) {
    let q, k, s, r;
    v[0] = 0;
    z[0] = -INF;
    z[1] = INF;
    for (q = 0; q < length; q++) {
        f[q] = grid[offset + q * stride];
    }
    for (q = 1, k = 0, s = 0; q < length; q++) {
        do {
            r = v[k];
            s = (f[q] - f[r] + q * q - r * r) / (q - r) / 2;
        } while (s <= z[k] && --k > -1);
        k++;
        v[k] = q;
        z[k] = s;
        z[k + 1] = INF;
    }
    for (q = 0, k = 0; q < length; q++) {
        while (z[k + 1] < q) {
            k++;
        }
        r = v[k];
        grid[offset + q * stride] = f[r] + (q - r) * (q - r);
    }
}
function edt(data, width, height, f, v, z) {
    for (let x = 0; x < width; x++) {
        edt1d(data, x, width, height, f, v, z);
    }
    for (let y = 0; y < height; y++) {
        edt1d(data, y * width, 1, width, f, v, z);
    }
}
//# sourceMappingURL=toSDF.js.map