import type { Node } from "./Node";
import type { ExactDecorativeProps, ExactLayoutProps, ViewStyleProps, LayoutNodeState } from "./styling";
/**
 * Basic building block of the UI. A node in a tree which is mutated by the layout algorithm.
 */
export declare class BaseView implements Node {
    readonly props: {
        onClick?(): void;
        style?: ViewStyleProps;
        testID?: string;
    };
    testID: string | null;
    next: Node | null;
    prev: Node | null;
    firstChild: Node | null;
    lastChild: Node | null;
    parent: Node | null;
    /**
     * Internal state of the node. It's public so that you can use it if you need to, but it's ugly
     * so that you don't forget it might break at any time.
     */
    _state: LayoutNodeState;
    /**
     * Should always be normalized.
     */
    _style: ExactDecorativeProps & ExactLayoutProps;
    constructor(props: {
        onClick?(): void;
        style?: ViewStyleProps;
        testID?: string;
    });
    add(node: Node): Node;
    remove(node: Node): void;
}
