import { normalizeLayoutProps, normalizeDecorativeProps, defaultLayoutNodeState } from "./styling";
/**
 * Basic building block of the UI. A node in a tree which is mutated by the layout algorithm.
 */
export class BaseView {
    props;
    testID;
    next = null;
    prev = null;
    firstChild = null;
    lastChild = null;
    parent = null;
    /**
     * Internal state of the node. It's public so that you can use it if you need to, but it's ugly
     * so that you don't forget it might break at any time.
     */
    _state = { ...defaultLayoutNodeState };
    /**
     * Should always be normalized.
     */
    _style;
    constructor(props) {
        this.props = props;
        this.testID = props.testID ?? null;
        this._style = normalizeDecorativeProps(normalizeLayoutProps(props.style ?? {}));
    }
    add(node) {
        node.parent = this;
        if (this.firstChild === null) {
            this.firstChild = node;
            this.lastChild = node;
        }
        else {
            if (this.lastChild === null) {
                throw new Error("Last child must be set.");
            }
            node.prev = this.lastChild;
            this.lastChild.next = node;
            this.lastChild = node;
        }
        return node;
    }
    remove(node) {
        // Check if node is a child of this node.
        if (node.parent !== this) {
            console.warn("Node is not a child of this node.");
        }
        if (node.prev !== null) {
            node.prev.next = node.next;
        }
        if (node.next !== null) {
            node.next.prev = node.prev;
        }
        if (this.firstChild === node) {
            this.firstChild = node.next;
        }
        if (this.lastChild === node) {
            this.lastChild = node.prev;
        }
        node.prev = null;
        node.next = null;
        node.parent = null;
    }
}
//# sourceMappingURL=BaseView.js.map