import { BaseView } from "./BaseView";
import type { BlurHandler, FocusHandler, KeyDownHandler, KeyPressHandler, KeyUpHandler, LayoutHandler, MouseClickHandler, MouseDownHandler, MouseEnterHandler, MouseLeaveHandler, MouseMoveHandler, MouseUpHandler, ScrollHandler } from "./eventTypes";
import { UserEventType } from "./eventTypes";
import type { ViewStyleProps } from "./styling";
type UserEventTuple = [UserEventType.MouseClick, MouseClickHandler] | [UserEventType.MouseMove, MouseMoveHandler] | [UserEventType.MouseEnter, MouseEnterHandler] | [UserEventType.MouseLeave, MouseLeaveHandler] | [UserEventType.MouseDown, MouseDownHandler] | [UserEventType.MouseUp, MouseUpHandler] | [UserEventType.Scroll, ScrollHandler] | [UserEventType.KeyDown, KeyDownHandler] | [UserEventType.KeyUp, KeyUpHandler] | [UserEventType.KeyPress, KeyPressHandler] | [UserEventType.Focus, FocusHandler] | [UserEventType.Blur, BlurHandler] | [UserEventType.Layout, LayoutHandler];
/**
 * `BaseView` but with event listeners.
 */
export declare class View extends BaseView {
    readonly props: {
        onClick?(): void;
        onMouseEnter?(): void;
        onMouseLeave?(): void;
        style?: ViewStyleProps;
        testID?: string;
    };
    _eventListeners: Array<UserEventTuple>;
    /**
     * Controlled by `EventManager`. Needed for dispatching mouseEnter and mouseLeave events.
     */
    _isMouseOver: boolean;
    _scrolling: {
        xActive: boolean;
        xHovered: boolean;
        yActive: boolean;
        yHovered: boolean;
    };
    constructor(props: {
        onClick?(): void;
        onMouseEnter?(): void;
        onMouseLeave?(): void;
        style?: ViewStyleProps;
        testID?: string;
    });
    private onScroll;
    private onMouseEnter;
    private onMouseLeave;
    private handleMouseDownScrollStart;
    private handleMouseMoveScrollHovering;
}
export {};
