import { describe, expect, it } from "vitest";
import { layout } from "./layout";
import { Vec2 } from "../math/Vec2";
import { paint } from "./paint";
import * as fixtures from "../fixtures";
import { compose } from "./compose";
import { EventManager } from "../EventManager";
import { getByTestId } from "../utils/getByTestId";
import { UserEventType } from "./eventTypes";
describe("paint", () => {
    it("handles scroll", () => {
        const eventManager = new EventManager();
        const ui = new MockRenderer();
        const root = fixtures.displayAndOverflow();
        layout(root, null, new Vec2(1000, 1000));
        compose(ui, root);
        function scroll(delta, where) {
            eventManager.dispatchEvent({
                bubbles: false,
                capturable: true,
                delta: delta,
                position: where,
                type: UserEventType.Scroll,
            });
            eventManager.deliverEvents(root);
            compose(ui, root);
            paint(ui, root);
        }
        scroll(new Vec2(0, 100), new Vec2(10, 10));
        expect(getByTestId(root, "D-halfWidth")?._state.scrollY).toBe(10);
        scroll(new Vec2(100, 0), new Vec2(10, 10));
        expect(getByTestId(root, "D-halfWidth")?._state.scrollY).toBe(10);
        expect(getByTestId(root, "D-halfWidth")?._state.scrollX).toBe(10);
        scroll(new Vec2(0, 100), new Vec2(240, 150));
        expect(getByTestId(root, "D-outer")?._state.scrollY).toBe(70);
    });
});
class MockRenderer {
    rectangle(_color, _position, _size, _corners, _borderWidth, _borderColor, _clipStart, _clipSize, _clipCorners) { }
    render(_commandEncoder) { }
    text(_text, _position, _fontName, _fontSize, _color, _textAlignment, _clipStart, _clipSize, _options) { }
}
//# sourceMappingURL=paint.test.js.map