import { Vec2 } from "./Vec2";
import { Vec3 } from "./Vec3";
const EPSILON = 0.001;
/**
 * A 4-dimensional vector.
 */
export class Vec4 {
    x;
    y;
    z;
    w;
    constructor(x, y, z, w) {
        this.x = x;
        this.y = y;
        this.z = z;
        this.w = w;
    }
    add(other) {
        return new Vec4(this.x + other.x, this.y + other.y, this.z + other.z, this.w + other.w);
    }
    subtract(other) {
        return new Vec4(this.x - other.x, this.y - other.y, this.z - other.z, this.w - other.w);
    }
    length() {
        return Math.sqrt(this.x * this.x + this.y * this.y + this.z * this.z + this.w * this.w);
    }
    normalize() {
        const length = this.length();
        return new Vec4(this.x / length, this.y / length, this.z / length, this.w / length);
    }
    scale(scalar) {
        return new Vec4(this.x * scalar, this.y * scalar, this.z * scalar, this.w * scalar);
    }
    cross(other) {
        return new Vec4(this.y * other.z - this.z * other.y, this.z * other.x - this.x * other.z, this.x * other.y - this.y * other.x, 0);
    }
    dot(other) {
        return this.x * other.x + this.y * other.y + this.z * other.z + this.w * other.w;
    }
    distance(other) {
        return this.subtract(other).length();
    }
    lerp(other, t) {
        return this.add(other.subtract(this).scale(t));
    }
    xy() {
        return new Vec2(this.x, this.y);
    }
    zw() {
        return new Vec2(this.z, this.w);
    }
    xyz() {
        return new Vec3(this.x, this.y, this.z);
    }
    equalsEpsilon(other, epsilon) {
        return (Math.abs(this.x - other.x) < epsilon &&
            Math.abs(this.y - other.y) < epsilon &&
            Math.abs(this.z - other.z) < epsilon &&
            Math.abs(this.w - other.w) < epsilon);
    }
    equals(other) {
        return this.equalsEpsilon(other, EPSILON);
    }
    data() {
        return [this.x, this.y, this.z, this.w];
    }
    toString() {
        return `(${this.x}, ${this.y}, ${this.z}, ${this.w})`;
    }
}
//# sourceMappingURL=Vec4.js.map